"""
 clusters_auto.py

  Cluster documents from their embeddings using the K-means algorithm.
  It reads the embeddings from a JSON file, applies K-means clustering,
  and saves the resulting clusters as a dictionary in another JSON file.

 minh.ha-duong@cnrs.fr, 2024-02    CC-BY-SA
"""

import json
import numpy as np
import matplotlib.pyplot as plt
from sklearn.cluster import KMeans
from sklearn.metrics import silhouette_score

NCLUSTERS = 8
IN_FILENAME = "../embeddings/embeddings.json"
OUT_FILENAME = "clusters.json"

# Read the embeddings from JSON file
with open(IN_FILENAME, "r", encoding="utf-8") as file:
    embeddings = {int(key): value for key, value in json.load(file).items()}

# Extract embeddings and corresponding docids
docids = list(embeddings.keys())
embedding_vectors = list(embeddings.values())

# Heuristics to choose the best number of clusters
max_potential_k = int(np.sqrt(len(embedding_vectors) / 2))

### Elbow method
inertias = []

for k in range(1, max_potential_k + 1):
    kmeans = KMeans(n_clusters=k, init="k-means++", random_state=0)
    kmeans.fit(embedding_vectors)
    inertias.append(kmeans.inertia_)

plt.figure(figsize=(8, 4))
plt.plot(range(1, max_potential_k + 1), inertias, "bo-")
plt.xlabel("Number of Clusters, k")
plt.ylabel("Inertia")
plt.title("Elbow Method For Optimal k")
plt.savefig("elbow_method.png")
plt.close()


### Silhouette Analysis

silhouette_scores = []
k_values = range(2, max_potential_k + 1)

for k in k_values:
    kmeans = KMeans(n_clusters=k, init="k-means++", random_state=0)
    cluster_labels = kmeans.fit_predict(embedding_vectors)
    silhouette_avg = silhouette_score(embedding_vectors, cluster_labels)
    silhouette_scores.append(silhouette_avg)

# Find the optimal k (max silhouette score)
optimal_k = k_values[silhouette_scores.index(max(silhouette_scores))]

# Find the second-best k
second_best_score = sorted(silhouette_scores, reverse=True)[
    1
]  # Sort and take the second score
second_best_k = k_values[silhouette_scores.index(second_best_score)]

# Plotting the Silhouette Analysis graph
plt.figure(figsize=(8, 4))
plt.plot(k_values, silhouette_scores, "bo-")
plt.xlabel("Number of Clusters, k")
plt.ylabel("Average Silhouette Score")
plt.title("Silhouette Analysis For Optimal k")

# Draw vertical lines for the max and second-best silhouette scores
plt.axvline(x=optimal_k, color="red", linestyle="--", label="Optimal k")
if optimal_k != second_best_k:  # Check if they are different to avoid overlapping lines
    plt.axvline(x=second_best_k, color="green", linestyle="--", label="Second Best k")

# Add legend to the plot
plt.legend()

plt.savefig("silhouette_analysis.png")  # Save the figure
plt.close()  # Close the plot to free up memory


# Perform clustering using KMeans
kmeans = KMeans(n_clusters=NCLUSTERS, init="k-means++", random_state=0)
cluster_labels = kmeans.fit_predict(np.array(embedding_vectors))

# Save the clusters as a dictionary
clusters = dict(zip(docids, cluster_labels.tolist()))

# Save the clusters dictionary as a JSON file
with open(OUT_FILENAME, "w", encoding="utf-8") as file:
    json.dump(clusters, file, ensure_ascii=False, indent=2)
