"""
Title Clusters

This script is used to generate titles for the clusters.

minh.ha-duong@cnrs.fr, 2024 CC-BY-SA
"""

import json
from openai import OpenAI
from diskcache import Cache

# MODEL = 'gpt-3.5-turbo'
MODEL = "gpt-4-turbo-preview"
IN_FILENAME = "../summarize/clusters_auto/summaries.txt"
OUT_FILENAME = "titles.json"
CACHE_DIR = f"cache_{MODEL}"

client = OpenAI()
cache = Cache(CACHE_DIR)


with open(IN_FILENAME, "r", encoding="utf-8") as file:
    summaries = json.load(file)


@cache.memoize()
def generate_title(summary):
    """
    Generates a title for a given text summary using the OpenAI API.
    This function is memoized using diskcache, so repeated calls with the same summary
    will return a cached result and not invoke the API again.

    :param summary: A string containing the text summary for which a title is to be generated.
    :return: A string containing the generated title.
    """
    messages = [
        {
            "role": "system",
            "content": "Give a short title to the text. Print only the title, nothing else.",
        },
        {"role": "user", "content": summary},
    ]
    response = client.chat.completions.create(messages=messages, model=MODEL)
    if response.choices and response.choices[0].message:
        return response.choices[0].message.content
    raise ValueError("Invalid response from the API.")


def sanitize(title):
    """
    Sanitize the title by removing double quotes and specific expressions.

    :param title: A string containing the title to be sanitized.
    :return: A sanitized string with specific expressions and double quotes removed.
    """
    title = title.replace('"', "")
    remove_expressions = [
        "Multidisciplinary ",
        "Approaches to ",
        "Approaches in ",
        "Interdisciplinary",
        "Research on",
        "Advancements in",
        "Advancing ",
        "Multifaceted ",
    ]
    for expr in remove_expressions:
        title = title.replace(expr, "")
    return title.strip()


try:
    titles = {
        idx: sanitize(generate_title(summary)) for idx, summary in summaries.items()
    }
finally:
    cache.close()

print(titles)

with open(OUT_FILENAME, "w", encoding="utf-8") as out_file:
    json.dump(titles, out_file, ensure_ascii=False, indent=4)
