"""
Classify.py

This script is used to automatically classify the abstracts.
It uses the cosine similarity to find the closest theme for each abstract, in the embedding space.
The classification is saved in a JSON file.

minh.ha-duong@cnrs.fr, 2024 CC-BY-SA
"""

import json
from sklearn.metrics.pairwise import cosine_similarity
import numpy as np

ABSTRACTS_EMBEDDINGS_FILENAME = "../embeddings/embeddings.json"
THEMES_EMBEDDINGS_FILENAME = "theme_embeddings.json"
OUT_FILENAME = "clusters.json"

# Load the theme embeddings
with open(THEMES_EMBEDDINGS_FILENAME, "r", encoding="utf-8") as file:
    theme_embeddings = json.load(file)

# Load the embeddings to be classified
with open(ABSTRACTS_EMBEDDINGS_FILENAME, "r", encoding="utf-8") as file:
    embeddings_to_classify = json.load(file)

# Convert embeddings to numpy arrays for cosine similarity calculation
theme_embedding_arrays = {
    theme: np.array(embedding) for theme, embedding in theme_embeddings.items()
}
embedding_to_classify_arrays = {
    doc_id: np.array(embedding) for doc_id, embedding in embeddings_to_classify.items()
}

# Classify each embedding
classified_embeddings = {}

for doc_id, embedding in embedding_to_classify_arrays.items():
    # Calculate similarity with each theme embedding
    similarities = {
        theme: cosine_similarity(
            embedding.reshape(1, -1), theme_embedding.reshape(1, -1)
        )[0][0]
        for theme, theme_embedding in theme_embedding_arrays.items()
    }

    # Find the theme with the highest similarity
    best_match_theme = max(similarities, key=similarities.get)
    classified_embeddings[doc_id] = best_match_theme

# Save the classification results to a JSON file
with open(OUT_FILENAME, "w", encoding="utf-8") as outfile:
    json.dump(classified_embeddings, outfile, indent=2, ensure_ascii=False)

print(f"Classification results saved into: {OUT_FILENAME}")
