"""
Theme Embeddings

This script is used to generate embeddings for the themes.
The embeddings are then saved in a JSON file.

minh.ha-duong@cnrs.fr, 2024 CC-BY-SA
"""

import json
import re
from openai import OpenAI

# Constants for embedding
MODEL = "text-embedding-3-large"
IN_FILENAME = "bilan.txt"
OUT_FILENAME = "theme_embeddings.json"

# Initialize the OpenAI client
client = OpenAI()


def parse_themes_from_file(filename):
    """
    Parses themes from a given file.

    This function reads a file line by line, identifying themes by a specific
    pattern ("Thème <number>.<number>") and collecting the associated text for each theme.

    Parameters:
    - filename: The name of the file to read from.

    Returns:
    A dictionary where the keys are the themes and the values are the associated text.
    """
    themes_dict = {}
    current_theme = None
    current_text = []

    with open(filename, "r", encoding="utf-8") as file:
        for line in file:
            if re.match(r"Thème \d+\.\d+", line):
                if current_theme and current_text:
                    themes_dict[current_theme] = " ".join(current_text)
                    current_text = []
                current_theme = line.strip()
            else:
                current_text.append(line.strip())

    if current_theme and current_text:
        themes_dict[current_theme] = " ".join(current_text)

    return themes_dict


def compute_embeddings(themes_dict):
    """
    Computes embeddings for each theme's text.

    Parameters:
    - themes_dict: A dictionary with theme names as keys and associated text as values.

    Returns:
    A dictionary where the keys are the themes and the values are the embeddings.
    """
    embeddings = {}
    for theme, text in themes_dict.items():
        response = client.embeddings.create(model=MODEL, input=text)
        embeddings[theme] = response.data[0].embedding
        print(f"Computed embedding for: {theme}")
        print(f"{text}")
    return embeddings


def save_embeddings_to_file(embeddings, filename):
    """
    Saves the embeddings to a JSON file.

    Parameters:
    - embeddings: A dictionary of embeddings to save.
    - filename: The name of the file to save the embeddings to.
    """
    with open(filename, "w", encoding="utf-8") as file:
        json.dump(embeddings, file, indent=2, ensure_ascii=False)
        print(f"Results saved into: {filename}")


# Main script execution
if __name__ == "__main__":
    themes = parse_themes_from_file(IN_FILENAME)
    theme_embeddings = compute_embeddings(themes)
    save_embeddings_to_file(theme_embeddings, OUT_FILENAME)
