"""
Coauthor Structure

This script is used to plot graphs showing the co-signature relation in the dataset.
For each graph, the authors are the nodes and the co-signature relation is the edges.
The graphs are generated for different values of min_appearances:
 With a higher parameter, only the most prolific authors are shown.

We disregards publications with too many signatures:
  They obscure the graph
  Presumably the social link between authors is weaker

Use an intermediate .dot file, as in-memory rendering with pygraphviz was slower

minh.ha-duong@cnrs.fr, 2024 CC-BY-SA
"""

import os
import json

import networkx as nx

IN_FILENAME = "../dataset/CIRED_dataset.json"
OUT_DIR = "graphs"

# Load the publication data from the JSON file
with open(IN_FILENAME, "r", encoding="utf-8") as file:
    publications_data = json.load(file)

# Create a dictionary to store the author information
authors = {}

# Iterate over each publication and extract the authors'names
TOOMANY_AUTHORS = 10
print("Reading JSON list of publications")
for pub in publications_data:
    if "authFullName_s" in pub:
        if len(pub["authFullName_s"]) > TOOMANY_AUTHORS:
            print(f"Skipping publication with more than {TOOMANY_AUTHORS} authors:")
            print(f"{pub['title_s'][0]}")
            print(f"{pub['authFullName_s']}\n")
            continue
        for name in pub["authFullName_s"]:
            if name not in authors:
                authors[name] = []
            authors[name].append(pub["authFullName_s"])

author_counts = {name: len(publications) for name, publications in authors.items()}

print("Flattening and deduplicating coauthors list\n")
for name, coauthors_list in authors.items():
    flat_coauthors = set()
    for coauthors in coauthors_list:
        flat_coauthors.update(coauthors)
    flat_coauthors.discard(name)
    authors[name] = flat_coauthors

min_appearances_values = [16, 12, 8, 4, 2, 1]

for min_appearances in min_appearances_values:
    G = nx.Graph()
    print(f"Creating graph for min_appearances = {min_appearances}")
    for name, coauthors_set in authors.items():
        if author_counts[name] >= min_appearances:
            G.add_node(
                name,
                fontname="bold",
                style="filled",
                fillcolor="lightblue2",
                fontsize=10 + author_counts[name] / 2,
            )
            for coauthor in coauthors_set:
                if author_counts[coauthor] >= min_appearances:
                    G.add_edge(name, coauthor, style="setlinewidth(0.05)")

    title = f"Collaborations, authors appearing at least {min_appearances} times"
    G.graph["graph"] = {
        "label": title,
        "labelloc": "t",
        "fontsize": 20,
        "fontcolor": "black",
        "fontname": "Helvetica",
    }

    dot_filename = f"{OUT_DIR}/collabs-min_appearances-{min_appearances}.dot"
    print(f"Writing graph to Graphviz format for min_appearances = {min_appearances}")
    nx.nx_agraph.write_dot(G, dot_filename)

    pdf_filename = f"{OUT_DIR}/collabs-min_appearances-{min_appearances}.pdf"
    SFDP_OPTIONS = "-Goverlap=false -Goutputorder=edgesfirst"
    command = f"sfdp {SFDP_OPTIONS} {dot_filename} -Tpdf -o {pdf_filename}"
    os.system(command)

    if os.path.isfile(dot_filename):
        os.remove(dot_filename)
