"""      names_mapping.py

Produce the substitution table to normalize the author name forms.
#TODO: this is too slow

minh.ha-duong@cnrs.fr, 2024 CC-BY-SA
"""

import json
import unicodedata
from collections import defaultdict
from fuzzywuzzy import fuzz

# Constants for file names
IN_FILENAME = "CIRED_publications_translated.json"
OUT_FILENAME = "names_mapping.json"

MISMATCHES = [
    "Liu Liu",
    "H. Liu",
    "Junguo Liu",
    "Hongxiao Liu",
    "Wenfeng Liu",
    "L Liu",
    "Anne Guillemot",
    "Laurent Li",
    "Tao Wang",
    "D Veneziano",
    "J Zhang",
    "K Wei",
    "W Wei",
    "P George",
    "T D’almeida",
    "P Walsh",
    "O Visser",
    "C Thomson",
    "M Smith",
    "Jean-Michel Fourniau",
    "D B Veneziano",
    "S Delgado",
    "C Joachim",
    "D Zavala",
    "C Morris",
    "L Mueller",
    "C J Johnson",
    "B Huang",
    "S M Schwartz",
    "J Espinoza",
    "Y H Li",
    "K Nakata",
    "M Mounier",
    "V Kumar",
    "E Molina",
    "C Navarro",
    "V Guillet",
]


def extract_author_names(publications):
    """Extract the authors' names from the publication data"""
    author_names = []
    for pub in publications:
        if "authFullName_s" in pub:
            for name in pub["authFullName_s"]:
                names = name.split(", ")  # Split the name by comma and space
                if len(names) > 1:
                    print(f"Split name field: {name}")  # Print the message
                author_names.extend(names)  # Add the split names to the list
    return author_names


def group_similar_names(author_names):
    """Group similar names using fuzzy matching."""
    name_groups = defaultdict(list)

    for name in author_names:
        matched = False
        for group in name_groups:
            if fuzz.token_set_ratio(name, group) > 85:
                name_groups[group].append(name)
                matched = True
                break

        if not matched:
            name_groups[name].append(name)

    return name_groups


def sorting_key(name):
    """Define the sorting key for names based on multiple criteria."""
    nfkd_form = unicodedata.normalize("NFKD", name)
    accent_count = sum(unicodedata.category(c) == "Mn" for c in nfkd_form)
    return (
        -len(name),  # prefer longer names, John Smith > J Smith
        -accent_count,  # prefer names with more accents, Vigué > Vigue
        -name.count("-"),  # prefer names with dashes, Jean-Charles > Jean Charles
        name[0].islower(),  # prefer names starting with uppercase, An > an
        name.count("."),  # prefer names without dots, An > A.
    )


def normalize_names(name_groups):
    """Return a dictionnary to map variant name forms to the preferred one."""
    result = {}

    for _, names in name_groups.items():
        if len(set(names)) == 1:  # No variant found
            continue

        names_sorted = sorted(
            names,
            key=sorting_key,
        )
        preferred_form = names_sorted[0]

        preferred_form = preferred_form.replace("\t", " ")
        preferred_form = " ".join(preferred_form.split())
        for name in names:
            if name != preferred_form:
                result[name] = preferred_form

    result = dict(sorted(result.items(), key=lambda item: item[1].split()[-1]))

    return result


def clean(names_map):
    """Hand-tune the list by removing and adding name variants."""
    for key in list(names_map):
        if key in MISMATCHES:
            print(f"Dropping {key}: {names_map[key]}")
            del names_map[key]
    names_map["V Viguié"] = "Vincent Viguié"
    return names_map


def main():
    """
    Orchestrate the normalization of author names in publication data.

    Load publication data, extract and group author names by similarity,
    normalize the names, exclude known mismatches, and save the canonical
    name mapping to a JSON file. The resulting dataset provides a consistent
    reference for author names.

    Steps:
    - Load publication data from JSON.
    - Extract and group similar author names using fuzzy matching.
    - Normalize names to their preferred form.
    - Remove known mismatches from the name mapping.
    - Save the clean mapping to a JSON file.
    """
    with open(IN_FILENAME, "r", encoding="utf-8") as file:
        publications = json.load(file)

    author_names = extract_author_names(publications)
    name_groups = group_similar_names(author_names)
    names_map = clean(normalize_names(name_groups))

    with open(OUT_FILENAME, mode="w", encoding="utf-8") as jsonfile:
        json.dump(names_map, jsonfile, ensure_ascii=False, indent=4)
    print(f"Saved names mapping to {OUT_FILENAME}")


if __name__ == "__main__":
    main()
