"""
scrape.py

Query the HAL API to obtain the list of CIRED records
Separate out those related to the homonymous CIRED conference
Save into two JSON files

minh.ha-duong@cnrs.fr, 2024 CC-BY-SA
"""

import json
import html
from typing import List, Dict

import requests

OUT_FILENAME = "CIRED_publications.json"
OTHER_FILENAME = "CIRED_conference.json"
HAL_API_URL = "https://api.archives-ouvertes.fr/search/"


def process_publications(publications: List[Dict]) -> None:
    """Process the publication records and save them into separate JSON files."""
    related_publications = []
    unrelated_cired_communications = []

    for pub in publications:
        pub["label_s"] = html.unescape(pub["label_s"])
        pub["producedDate_tdate"] = pub["producedDate_tdate"].split("T")[0]

        lab_acronym_present = (
            "labStructAcronym_s" in pub and "CIRED" in pub["labStructAcronym_s"]
        )
        cired_in_citation = "CIRED" in pub.get("label_s", "")

        if not lab_acronym_present and cired_in_citation:
            unrelated_cired_communications.append(pub)
        else:
            related_publications.append(pub)

    with open(OUT_FILENAME, "w", encoding="utf-8") as file:
        json.dump(related_publications, file, ensure_ascii=False, indent=4)

    with open(OTHER_FILENAME, "w", encoding="utf-8") as file:
        json.dump(unrelated_cired_communications, file, ensure_ascii=False, indent=4)

    print("CIRED HAL records from 2018 to 2023 saved successfully.")
    print("Unrelated CIRED conference communications saved for inspection.")


def get_publications(params: Dict) -> List[Dict]:
    """Retrieve the publication records from the HAL API."""
    try:
        response = requests.get(HAL_API_URL, params=params, timeout=60)
        response.raise_for_status()
        return response.json()["response"]["docs"]
    except requests.exceptions.Timeout:
        print("The request timed out. Please try again later.")
    except requests.exceptions.HTTPError as err:
        print(f"HTTP error occurred: {err}")
    except requests.exceptions.RequestException as e:
        print(f"An error occurred while requesting data: {e}")
    return []


def main() -> None:
    """Main function to orchestrate the retrieval and processing of publications."""
    fields = """
docid,halId_s,doiId_s,label_s,producedDate_tdate,authFullName_s,title_s,abstract_s,submitType_s,docType_s,labStructAcronym_s
"""
    params = {
        "q": "CIRED",
        "fq": "producedDateY_i:[2018 TO 2023]",
        "fl": fields.strip(),
        "wt": "json",
        "rows": 1500,
    }

    publications = get_publications(params)
    if publications:
        process_publications(publications)


if __name__ == "__main__":
    main()
