"""      translate.py

Translate titles and abstracts to English.
Deduplicate multi-language fields. 

minh.ha-duong@cnrs.fr, 2024 CC-BY-SA
"""

import json
from langdetect import detect
from openai import OpenAI
from diskcache import Cache

IN_FILENAME = "CIRED_publications.json"
OUT_FILENAME = "CIRED_publications_translated.json"

# Directory for diskcache
CACHE_DIR = "translation_cache"

# Initialize the diskcache Cache
cache = Cache(CACHE_DIR)

with open(IN_FILENAME, "r", encoding="utf-8") as file:
    publications = json.load(file)

# Initialize the OpenAI client
MODEL = "gpt-4-turbo-preview"
client = OpenAI()

SYSTEM_PROMPT = """
Translate the text into English if necessary, otherwise just reprint the text as given.
If there are multiple versions of the content in different languages, deduplicate.
"""


@cache.memoize()  # Cache never expires
def translate(text):
    """Use ChatGPT to translate and deduplicate."""
    print(f"Translating: {text}")
    messages = [
        {
            "role": "system",
            "content": SYSTEM_PROMPT,
        }
    ]
    messages.append({"role": "user", "content": text})

    response = client.chat.completions.create(messages=messages, model=MODEL)
    if response.choices and response.choices[0].message:
        return response.choices[0].message.content
    raise ValueError(f"Invalid response from the API.\n{response}")


try:
    # The main loop for translation with cache checking
    for pub in publications:
        if "title_s" in pub:
            title = pub["title_s"][0]
            if detect(title) != "en":
                title = translate(title)
            pub["title_s"] = [title]
            print(pub["title_s"])

        if "abstract_s" in pub:
            abstract = pub["abstract_s"][0]
            if detect(abstract) != "en":
                abstract = translate(abstract)
            pub["abstract_s"] = [abstract]
            print(pub["abstract_s"])
        else:
            print("No abstract")
        print()
finally:
    print("Closing the cache...")
    cache.close()
    print("Cache closed.")

# Save the updated data to a new JSON file
with open(OUT_FILENAME, "w", encoding="utf-8") as file:
    json.dump(publications, file, ensure_ascii=False, indent=4)
