"""
  Compute embeddings of the abstracts of CIRED scientific production 2018-2023

  minh.ha-duong@cnrs.fr, 2024-02    CC-BY-SA

"""

import json
from openai import OpenAI
from gensim.utils import simple_preprocess
from gensim.parsing.preprocessing import STOPWORDS
from diskcache import Cache

DEVELOPMENT_MODE = False
DEVELOPMENT_MODE_LIMIT = 5

IN_FILENAME = "../dataset/CIRED_dataset.json"
OUT_FILENAME = "embeddings.json"

CACHE_DIR = "embeddings_cache"
cache = Cache(CACHE_DIR)

if DEVELOPMENT_MODE:
    OUT_FILENAME = "tmp.json"

MODEL = "text-embedding-3-large"
client = OpenAI()

with open(IN_FILENAME, "r", encoding="utf-8") as file:
    publications_data = json.load(file)


custom_stopwords = [
    "the",
    "this",
    "we",
    "in",
    "paper",
    "model",
    "models",
    "study",
    "studies",
    "based",
    "use",
    "results",
    "analysis",
    "es",
    "new",
    "research",
    "different",
    "change",
    "ae",
    "million",
    "data",
    "effects",
    "article",
    "compared",
    "average",
    "higher",
    "influence",
    "approach",
    "particularly",
]

# Extend the default stop words with custom stop words
stop_words = STOPWORDS.union(set(custom_stopwords))


# Normalize and preprocess the tokens
def normalize_token(token):
    normalization_dict = {
        "french": "france",
        "France": "france",
        "india": "india",
        "India": "india",
        "indian": "india",
        "policy": "policy",
        "policies": "policy",
        "network": "network",
        "networks": "network",
        "scenario": "scenario",
        "scenarios": "scenario",
    }
    return normalization_dict.get(token, token)


def preprocess_text(text):
    tokens = simple_preprocess(text, deacc=True)
    normalized_tokens = [normalize_token(token) for token in tokens]
    filtered_tokens = [token for token in normalized_tokens if token not in stop_words]
    return filtered_tokens


publications_with_abstract = [
    pub
    for pub in publications_data
    if "abstract_s" in pub and pub["abstract_s"] and pub["abstract_s"][0].strip()
]


@cache.memoize()
def get_embedding(text):
    response = client.embeddings.create(model=MODEL, input=[text])
    return response.data[0].embedding


dev_mode_counter = 0
try:
    for pub in publications_with_abstract:
        # Tokenize the title and abstract
        pub["title_tokens"] = preprocess_text(pub["title_s"][0])
        pub["abstract_tokens"] = preprocess_text(pub["abstract_s"][0])

        # Use the abstract tokens for embedding
        text = " ".join(pub["abstract_tokens"])
        pub["embedding"] = get_embedding(text)
        print(pub["docid"], end=" ", flush=True)

        dev_mode_counter += 1
        if DEVELOPMENT_MODE and dev_mode_counter >= DEVELOPMENT_MODE_LIMIT:
            break
finally:
    print("Closing the cache...")
    cache.close()
    print("Cache closed.")

embeddings = {
    pub["docid"]: pub["embedding"]
    for pub in publications_with_abstract
    if "embedding" in pub
}


with open(OUT_FILENAME, "w", encoding="utf-8") as file:
    json.dump(embeddings, file, indent=2, ensure_ascii=False)
    print(f"\nResults saved into: {OUT_FILENAME}")
