"""
Analyse

This script is used to analyse the facets of CIRED scientific production.

minh.ha-duong@cnrs.fr, 2024 CC-BY-SA
"""

import pandas as pd
import numpy as np

pd.set_option("display.max_rows", None)
pd.set_option("display.max_columns", None)
pd.set_option("display.width", None)

IN_FILENAME = "facets.tsv"
OUT_FILENAME = "facets_analysis.csv"

SDG_NAMES = {
    1: "1 No poverty",
    2: "2 Zero hunger",
    3: "3 Good health and well-being",
    4: "4 Quality education",
    5: "5 Gender equality",
    6: "6 Clean water and sanitation",
    7: "7 Affordable and clean energy",
    8: "8 Decent work and economic growth",
    9: "9 Industry, innovation, and infrastructure",
    10: "10 Reduced inequalities",
    11: "11 Sustainable cities and communities",
    12: "12 Responsible consumption and production",
    13: "13 Climate action",
    14: "14 Life below water",
    15: "15 Life on land",
    16: "16 Peace, justice, and strong institutions",
    17: "17 Partnerships for the goals",
}

DISCIPLINE_NORM = {
    "agricultural sciences": "agricultural science",
    "agronomy": "agricultural science",
    "agriculture": "agricultural science",
    "political science": "policy studies",
    "political sciences": "policy studies",
    "policy analysis": "policy studies",
    "policy studies": "policy studies",
    "international relations": "policy studies",
    "social policy": "policy studies",
    "transportation studies": "transport studies",
    "climate science": "climatology",
    "energy science": "energy studies",
    "environmental studies": "environmental science",
    "environmental management": "environmental science",
    "social science": "sociology",
    "social sciences": "sociology",
    "urban planning": "urban studies",
    "environmental economics": "economics",
    "ecological economics": "economics",
    "agricultural economics": "economics",
    "urban economics": "economics",
    "political economy": "economics",
    "international trade": "economics",
    "spatial econometrics": "economics",
    "development studies": "economics",
    "finance": "economics",
    "remote sensing": "geography",
    "technology studies": "engineering",
    "energy engineering": "engineering",
    "transportation engineering": "engineering",
    "information technology": "engineering",
    "ethics": "philosophy",
    "epistemology": "philosophy",
    "social network analysis": "sociology",
    "statistics": "data science",
    "HCI and visualization": "data science",
    "business studies": "management sciences",
    "operations research": "management sciences",
    "accounting": "management sciences",
    "management": "management sciences",
    "multi- and interdisciplinary research": "sustainability science",
    "environmental health": "public health",
}

AREA_NORM = {
    "EU": "Europe",
    "European Union": "Europe",
    "Paris": "Paris area",
    "Paris region": "Paris area",
    "Paris (France)": "Paris area",
    "Paris, France": "Paris area",
    "Paris metropolitan area": "Paris area",
    "Greater Paris metropolitan area": "Paris area",
    "Ile de France": "Paris area",
    "Ile-de-France": "Paris area",
    "Paris region, France": "Paris area",
    "Île-de-France": "Paris area",
    "France (French students)": "France",
    "Mariño watershed, Peruvian Andes": "Mariño watershed, Peru",
    "Mariño watershed (Peru)": "Mariño watershed, Peru",
    "Peruvian Andes": "Peru",
    "Mediterranean France": "French Mediterranean",
    "Europe (France, Germany, Norway, Sweden)": "France, Germany, Norway, Sweden",
    "Europe (UK, France)": "France, UK",
    "France and UK": "France, UK",
    "Europe (France)": "France",
}

TIME_SCOPE_NORM = {
    "1960-1990s": "recent past",
}

# Read the table data from the TSV file
df = pd.read_csv(
    IN_FILENAME,
    delimiter="\t",
    dtype={
        "Main SDG": pd.Int64Dtype(),
        "Other SDG": pd.Int64Dtype(),
        "Third SDG": pd.Int64Dtype(),
    },
    na_values=["NA", "N/A", "NaN", ""],
)

df["Main discipline"].replace(DISCIPLINE_NORM, inplace=True)
df["Secondary discipline"].replace(DISCIPLINE_NORM, inplace=True)
df["Area"].replace(AREA_NORM, inplace=True)
df["Time scope"].replace(TIME_SCOPE_NORM, inplace=True)


# Basic analysis
print("Number of rows:", len(df))
print("Columns:", df.columns)
print("")


def analyse_sdgs(dataframe):
    """Count the SDG occurences. Returns a dataframe."""

    main_counts = dataframe["Main SDG"].value_counts(dropna=False)
    other_counts = dataframe["Other SDG"].value_counts(dropna=False)
    third_counts = dataframe["Third SDG"].value_counts(dropna=False)

    combined_counts = main_counts.add(other_counts, fill_value=0).add(
        third_counts, fill_value=0
    )

    weighted_counts = main_counts.add(other_counts * 0.5, fill_value=0).add(
        third_counts * (1 / 3), fill_value=0
    )

    weighted_counts = np.ceil(weighted_counts).astype(int)

    counts_df = pd.DataFrame(
        {
            "Main SDG": main_counts,
            "Other SDG": other_counts,
            "Third SDG": third_counts,
            "Combined": combined_counts,
            "Weighted": weighted_counts,
        }
    )

    counts_df = counts_df.fillna(0)

    if pd.NA in counts_df.index:
        counts_df = counts_df.drop(pd.NA)

    counts_df = counts_df.astype(int)
    counts_df = counts_df.sort_values(by="Combined", ascending=False)

    counts_df.rename(index=SDG_NAMES, inplace=True)
    return counts_df


print("SDG Counts Table:")
print(analyse_sdgs(df))


def analyse_disciplines(dataframe):
    """Analyze the discipline counts from the dataframe."""

    main_discipline_counts = dataframe["Main discipline"].value_counts(dropna=True)
    secondary_discipline_counts = dataframe["Secondary discipline"].value_counts(
        dropna=True
    )

    weighted_discipline_counts = main_discipline_counts.add(
        secondary_discipline_counts * 0.5, fill_value=0
    )

    weighted_discipline_counts = np.ceil(weighted_discipline_counts).astype(int)

    discipline_counts_df = pd.DataFrame(
        {
            "Main Discipline": main_discipline_counts,
            "Secondary Discipline": secondary_discipline_counts,
            "Weighted": weighted_discipline_counts,
        }
    )

    discipline_counts_df = discipline_counts_df.fillna(0).astype(int)

    if pd.NA in discipline_counts_df.index:
        discipline_counts_df = discipline_counts_df.drop(pd.NA)

    discipline_counts_df.sort_values(by="Weighted", ascending=False, inplace=True)

    return discipline_counts_df


print("Discipline Counts Table:")
print(analyse_disciplines(df))


method_counts = df["Method"].value_counts(dropna=False)
print("Method Counts:")
print(method_counts)


area_counts = df["Area"].value_counts(dropna=True)
print("\nArea Counts:")
print(area_counts)


filtered_time_scope = df["Time scope"].dropna()
filtered_time_scope = filtered_time_scope[filtered_time_scope != "context"]
time_scope_counts = filtered_time_scope.value_counts(dropna=False)
print("\nTime Scope Counts:")
print(time_scope_counts)

# %% Save the dataframe with all the applied substitutions and counts to a new CSV file

df.to_csv(OUT_FILENAME, index=False)
