"""
reference.py

Add citations to the summary of our research activities.

minh.ha-duong@cnrs.fr, 2024 CC-BY-SA
"""


import argparse
import re
import json
import numpy as np
from openai import OpenAI
from sklearn.metrics.pairwise import cosine_similarity


# Set up argument parser
parser = argparse.ArgumentParser(description="Generate t-SNE plot for clusters.")
parser.add_argument(
    "--clusters",
    type=str,
    choices=["auto", "manual"],
    required=True,
    help="Specify clustering mode: auto or manual",
)
args = parser.parse_args()

MODEL = "text-embedding-3-large"

EMBEDDINGS_FILENAME = "../embeddings/embeddings.json"
PUBLICATIONS_FILENAME = "../dataset/CIRED_dataset.json"
SUMMARIES_FILENAME = f"clusters_{args.clusters}/summaries.txt"
OUT_FILENAME = f"clusters_{args.clusters}/summaries+references.json"
REFERENCES_COUNT = 3

# Initialize the OpenAI client
client = OpenAI()


# Read the document ID to citations
with open(PUBLICATIONS_FILENAME, "r", encoding="utf-8") as file:
    publications = json.load(file)
    labels = {pub["docid"]: pub["label_s"] for pub in publications}

# Read the summaries
with open(SUMMARIES_FILENAME, "r", encoding="utf-8") as file:
    summaries = json.load(file)

# Read the precomputed embeddings
# FIXME: This loads publications embeddings. We should filter on pubType_s = "ART"
with open(EMBEDDINGS_FILENAME, "r", encoding="utf-8") as file:
    articles_embeddings = json.load(file)


def split_into_paragraphs(summary):
    """
    Splits a given summary text into paragraphs.
    This function specifically deals with the case
    where ### headers are followed by a single newline character.

    Parameters:
    - summary (str): The summary text to be split into paragraphs.

    Returns:
    - list: A list of paragraphs from the summary.
    """
    paragraphs = re.split(r"\n\n|\n(?=\S)", summary)
    updated_paragraphs = []
    for paragraph in paragraphs:
        if paragraph.startswith("###"):
            split_index = min(paragraph.find("\n"), paragraph.find("\n\n"))
            if split_index > 0:
                updated_paragraphs.append(paragraph[:split_index])
                updated_paragraphs.append(paragraph[split_index:].strip())
            else:
                updated_paragraphs.append(paragraph)
        else:
            updated_paragraphs.append(paragraph)
    return updated_paragraphs


def compute_embeddings(texts):
    """
    Computes embeddings for a list of texts using a pre-defined model.

    Parameters:
    - texts (list of str): The list of texts to compute embeddings for.

    Returns:
    - list: A list of embeddings corresponding to the input texts.
    """
    response = client.embeddings.create(model=MODEL, input=texts)
    return [entry.embedding for entry in response.data]


def find_closest_articles(paragraph_embedding, article_embeddings, k=REFERENCES_COUNT):
    """
    Finds the closest articles to a given paragraph embedding.

    Parameters:
    - paragraph_embedding (list): Embedding of the paragraph for which to find the closest articles.
    - article_embeddings (dict): A dictionary of article embeddings.
    - k (int, optional): The number of closest articles to retrieve. Defaults to REFERENCES_COUNT.

    Returns:
    - list: A list of keys (article IDs) of the closest articles.
    """
    similarity_scores = cosine_similarity(
        [paragraph_embedding], list(article_embeddings.values())
    )
    closest_indices = np.argsort(similarity_scores[0])[-k:][
        ::-1
    ]  # Reverse to get top articles first
    closest_articles = [
        list(article_embeddings.keys())[index] for index in closest_indices
    ]
    return closest_articles


def replace_docids_with_citations(references, labels):
    """
    Replaces document IDs in a list with their corresponding citations.

    Parameters:
    - references (list): A list of document IDs that need to be replaced with citations.
    - labels (dict): A dictionary mapping document IDs to citations.

    Returns:
    - str: A string of citations separated by line breaks.
    """
    citations = []
    for docid in references:
        # Replace docid with the actual citation from the dictionary
        citation = labels.get(docid, f"[{docid}]")  # Keep [docid] if not found
        citations.append(citation)
    return "\n".join(citations)  # Join citations with a line break instead of space


paragraphs_info = {}

# First pass: process each summary to extract paragraphs and mark those that need references
for topic, summary in summaries.items():
    paragraphs = split_into_paragraphs(summary)

    paragraphs_info[topic] = {
        "paragraphs": paragraphs,
        "embeddings": [],
        "reference_flags": [False] * len(paragraphs),  # Initialize flags as False
    }

    for i, paragraph in enumerate(paragraphs):
        if paragraph.startswith("###"):
            # Start adding references to paragraphs that follow headers
            paragraphs_info[topic]["reference_flags"][i + 1] = True

# Compute embeddings for paragraphs that need references
for topic in paragraphs_info:
    paragraphs_to_embed = [
        paragraph
        for paragraph, flag in zip(
            paragraphs_info[topic]["paragraphs"],
            paragraphs_info[topic]["reference_flags"],
        )
        if flag
    ]
    if paragraphs_to_embed:
        paragraphs_info[topic]["embeddings"] = compute_embeddings(paragraphs_to_embed)

# Second pass: select references for each topic and add them to the paragraphs
updated_summaries = {}

# Get the embeddings for all articles to use in reference selection
all_article_embeddings = np.array(list(articles_embeddings.values()))
all_article_keys = list(articles_embeddings.keys())  # Store the keys to maintain order


# Select references for paragraphs that need them
paragraphs_references = {}

for topic, info in paragraphs_info.items():
    if info["embeddings"]:
        similarity_matrix = cosine_similarity(
            info["embeddings"], all_article_embeddings
        )

        # Initialize a list to hold references for each paragraph
        topic_paragraphs_references = [[] for _ in info["embeddings"]]

        # Initialize a set to keep track of used article indices
        used_indices = set()

        # We continue iterating until all paragraphs have the required number of references
        while any(len(refs) < REFERENCES_COUNT for refs in topic_paragraphs_references):
            for paragraph_idx, paragraph_embedding in enumerate(info["embeddings"]):
                # Skip if this paragraph already has enough references
                if len(topic_paragraphs_references[paragraph_idx]) >= REFERENCES_COUNT:
                    continue

                # Get the index of the top unused article
                unused_indices = [
                    i for i in range(len(all_article_keys)) if i not in used_indices
                ]
                # If all articles are used up, we cannot assign more references
                if not unused_indices:
                    break

                # Select the article with the highest similarity that hasn't been used yet
                best_unused_idx = unused_indices[
                    np.argmax(similarity_matrix[paragraph_idx, unused_indices])
                ]
                best_unused_article = all_article_keys[best_unused_idx]

                # Add the reference to the paragraph list
                topic_paragraphs_references[paragraph_idx].append(best_unused_article)
                # Mark the article as used
                used_indices.add(best_unused_idx)

        # Store the references for each paragraph
        for paragraph_idx, refs in enumerate(topic_paragraphs_references):
            paragraphs_references[(topic, paragraph_idx)] = refs


for topic in summaries.keys():
    updated_summary = ""
    paragraph_count = len(paragraphs_info[topic]["paragraphs"])
    reference_paragraph_idx = 0  # Reset reference index for each topic

    for i in range(paragraph_count):
        paragraph = paragraphs_info[topic]["paragraphs"][i]
        updated_summary += paragraph + "\n\n"

        if paragraphs_info[topic]["reference_flags"][i]:
            # Get the precomputed closest articles for the current paragraph
            closest_articles = paragraphs_references[(topic, reference_paragraph_idx)]
            reference_paragraph_idx += (
                1  # Move to the next paragraph that needs references
            )

            # Convert docids to citations
            references_str = replace_docids_with_citations(closest_articles, labels)

            updated_summary += "References:\n" + references_str + "\n\n"

    updated_summaries[topic] = updated_summary.rstrip("\n\n")

# Save the updated summaries with references
with open(OUT_FILENAME, "w", encoding="utf-8") as file:
    json.dump(updated_summaries, file, indent=2, ensure_ascii=False)
