"""
summarize.py

Asks ChatGPT to write the summary of our research activities.
The guidelines are taken from HCERS reporting template part 3.

Consider only ART type publications.

The abstracts are processed in batches,
each batch corresponding to a scientific theme defined before by clustering.

Remember to invalidate the cache if changing the prompt.

minh.ha-duong@cnrs.fr, 2024 CC-BY-SA
"""

import argparse
import json
from openai import OpenAI
from diskcache import Cache

# Set up argument parser
parser = argparse.ArgumentParser(description="Generate t-SNE plot for clusters.")
parser.add_argument(
    "--clusters",
    type=str,
    choices=["auto", "manual"],
    required=True,
    help="Specify clustering mode: auto or manual",
)
args = parser.parse_args()

MODEL = "gpt-4-turbo-preview"
CONTEXT_WINDOW = 128000
IN_FILENAME = "../dataset/CIRED_dataset.json"
CLUSTERS_FILENAME = f"../clusters_{args.clusters}/clusters.json"
OUT_FILENAME = f"clusters_{args.clusters}/summaries.txt"
CACHE_DIR = "chatgpt_cache"

cache = Cache(CACHE_DIR)


with open(IN_FILENAME, "r", encoding="utf-8") as file:
    publications_data = json.load(file)
    abstracts = {
        pub["docid"]: pub["abstract_s"][0]
        for pub in publications_data
        if "abstract_s" in pub and pub["docType_s"] == "ART"
    }

with open(CLUSTERS_FILENAME, "r", encoding="utf-8") as file:
    clusters = json.load(file)
    valid_cluster_ids = {
        cluster_id for docid, cluster_id in clusters.items() if docid in abstracts
    }

client = OpenAI()

# TODO  Reengineer the prompt for proper summary. Maybe use a chain of thought?
SYSTEM_PROMPT = """
Please generate a cohesive summary of the research themes your group has addressed during the reference period. This summary should:

    Place your research themes within the international context.
    Highlight significant noteworthy scientific advancements.
    Explain how these notable scientific facts respond to scientific, technological, cultural, economic, or societal challenges.
    Ensure the summary is substantial but remains of a reasonable length.

For this task, you will be reviewing a collection of abstracts related to your group's research. Extract key information to craft a narrative that meets the above criteria.
Format the answer in Markdown.
After an introductory paragraph, give each of part a subtitle marked with a level three heading "### ".
Start the narrative with "The research themes of this group are"
"""


@cache.memoize()
def summarize(collection):
    """Ask ChatGPT to summarize. Warning: the function can be costly and slow."""
    print("Memoized version not found in cache, generating summary...")
    messages = [
        {"role": "system", "content": SYSTEM_PROMPT},
        {"role": "user", "content": str(collection)},
    ]

    word_count = sum(len(word) for word in str(messages).split())
    token_count = word_count * 4 / 3
    if token_count > CONTEXT_WINDOW:
        raise ValueError(f"Token count {token_count} exceeds the GPT-4 limit.")

    try:
        response = client.chat.completions.create(messages=messages, model=MODEL)
        print(response)
        summary = response.choices[0].message.content
        return summary
    except Exception as e:
        return f"An error occurred during summarization: {e}"


# Generate summary for each cluster
cluster_summaries = {}
try:
    for cluster_id in valid_cluster_ids:
        # Retrieve the abstracts belonging to the current cluster
        cluster_abstracts = [
            abstracts[docid]
            for docid, cluster in clusters.items()
            if cluster == cluster_id and docid in abstracts
        ]
        print(cluster_id)
        cluster_summary = summarize(cluster_abstracts)
        cluster_summaries[cluster_id] = cluster_summary
finally:
    cache.close()

# Save the summaries to a file with JSON du
with open(OUT_FILENAME, "w", encoding="utf-8") as file:
    json.dump(cluster_summaries, file, ensure_ascii=False, indent=2)
