"""
todocx.py

Create a Microsoft Word document from JSON-formatted summaries and references,
incorporating figures and organizing content into a structured format.

Features:
- Parses JSON for summaries and references.
- Inserts and resizes figures to fit document formatting.
- Organizes content with headings and subheadings.
- Applies special formatting for references.
- Generates a comprehensive report with title, authorship, and date.

The module streamlines the presentation of research findings in a professional document layout.
Ideal for academic and research purposes, enhancing the clarity and appeal of presented findings.

minh.ha-duong@cnrs.fr, 2024 CC-BY-SA
"""

import argparse
import os
import json
import re
from datetime import datetime
from docx import Document


# Set up argument parser
parser = argparse.ArgumentParser(description="Generate t-SNE plot for clusters.")
parser.add_argument(
    "--clusters",
    type=str,
    choices=["auto", "manual"],
    required=True,
    help="Specify clustering mode: auto or manual",
)
args = parser.parse_args()

TEXT_FILENAME = f"../summarize/clusters_{args.clusters}/summaries+references.json"
TITLES_FILENAME = f"../clusters_{args.clusters}/titles.json"
PLOTS_DIR = f"../wordclouds/clusters_{args.clusters}/"
CLUSTERS_FIGURE = f"../tsne_plot/tsne_plot-clusters_{args.clusters}.png"
OUT_FILENAME = f"clusters_{args.clusters}/summaries+references.docx"


with open(TEXT_FILENAME, "r", encoding="utf-8") as file:
    updated_summaries = json.load(file)

titles = None  # pylint: disable=invalid-name
if os.path.isfile(TITLES_FILENAME):
    with open(TITLES_FILENAME, "r", encoding="utf-8") as titles_file:
        titles = json.load(titles_file)


def add_figure(document, figure_path):
    """Add a figure to the document, paragraph-wide."""
    try:
        run = document.add_paragraph().add_run()
        picture = run.add_picture(figure_path)

        original_width, original_height = picture.width, picture.height
        # Get the width of the paragraph
        paragraph_width = (
            document.sections[0].page_width
            - document.sections[0].left_margin
            - document.sections[0].right_margin
        )

        # Calculate the scale factor based on the width
        scale_factor = paragraph_width / original_width

        # Scale the picture width and height
        picture.width = int(original_width * scale_factor)
        picture.height = int(original_height * scale_factor)
    except FileNotFoundError:
        print(f"WARNING: Figure file not found: {figure_path}")


# Create a new Document
doc = Document()

doc.add_heading("Automatic analysis of CIRED 2018-2023 publications", 0)

doc.add_paragraph("Prepared by Minh Ha-Duong <minh.ha-duong@cnrs.fr>")

today_date = datetime.today().strftime("%Y-%m-%d")
doc.add_paragraph(f"Generated on {today_date}")

if args.clusters == "manual":
    doc.add_paragraph("Axes and themes defined manually using the February 19 draft.")

if args.clusters == "auto":
    doc.add_paragraph("Axes and themes defined automaticaly.")

doc.add_heading("All our abstracts in HAL for the period")

add_figure(doc, f"{PLOTS_DIR}all.png")

doc.add_heading("Abstracts clustering used in this report")
add_figure(doc, CLUSTERS_FIGURE)

# Loop through the summaries and add them to the document, sorted alphabetically by topic
for index, topic in enumerate(sorted(updated_summaries.keys())):
    summary = updated_summaries[topic]
    heading_text = f"{topic}. {titles.get(topic)}" if titles else topic
    print(heading_text)

    if args.clusters == "manual":
        if index == 0:
            doc.add_heading("Axe 1", level=1)
        if index == 3:
            doc.add_heading("Axe 2", level=1)
        if index == 7:
            doc.add_heading("Axe 3", level=1)

    # Add a level 2 heading for the topic
    doc.add_heading(heading_text, level=2)

    add_figure(doc, f"{PLOTS_DIR}{topic}.png")

    # Add the summary text with references
    paragraphs = summary.split("\n\n")
    for paragraph in paragraphs:
        if paragraph.startswith("###"):
            # Translate Level 3 markdown headers to Level 3 headers in the Word document
            heading_text = re.sub(r"### (.*)", r"\1", paragraph)
            doc.add_heading(heading_text, level=3)
        elif paragraph.startswith("References:"):
            # Add references in italic style
            ref_paragraph = doc.add_paragraph()
            ref_run = ref_paragraph.add_run(paragraph)
            ref_run.italic = True
        else:
            # Add normal summary text
            doc.add_paragraph(paragraph)


# Save the document
doc.save(OUT_FILENAME)
