"""
TSNE Plot

This script is used to generate a t-SNE plot for the clusters.

minh.ha-duong@cnrs.fr, 2024 CC-BY-SA
"""

import os
import argparse
import json
import numpy as np
import matplotlib.pyplot as plt
import matplotlib
from sklearn.manifold import TSNE


parser = argparse.ArgumentParser(description="Generate t-SNE plot for clusters.")
parser.add_argument(
    "--clusters",
    type=str,
    choices=["auto", "manual"],
    required=True,
    help="Specify clustering mode: auto or manual",
)
args = parser.parse_args()

EMBEDDINGS_FILENAME = "../embeddings/embeddings.json"
CLUSTERS_FILENAME = f"../clusters_{args.clusters}/clusters.json"
TITLES_FILENAME = f"../clusters_{args.clusters}/titles.json"

PLOT_FILENAME = f"tsne_plot-clusters_{args.clusters}.png"


with open(EMBEDDINGS_FILENAME, "r", encoding="utf-8") as file:
    embeddings = {int(key): value for key, value in json.load(file).items()}

with open(CLUSTERS_FILENAME, "r", encoding="utf-8") as file:
    print(f"Reading clusters from {CLUSTERS_FILENAME}")
    clusters = json.load(file)

titles = None  # pylint: disable=invalid-name
if os.path.isfile(TITLES_FILENAME):
    with open(TITLES_FILENAME, "r", encoding="utf-8") as file:
        titles = json.load(file)

docids = list(embeddings.keys())
embedding_vectors = list(embeddings.values())

# Keep as a np.array because we use boolean indexing to define cluster_points
cluster_labels = np.array([clusters[str(docid)] for docid in docids])

display_labels = {}
for label in sorted(set(cluster_labels)):
    if args.clusters == "manual":
        if len(label) > 60:
            display_labels[label] = f"{label[6:60]}..."
        else:
            display_labels[label] = label[6:]
    else:
        if titles and str(label) in titles:
            display_labels[label] = titles[str(label)]
        else:
            display_labels[label] = f"Cluster {label}"


tsne = TSNE(n_components=2, random_state=0)
tsne_results = tsne.fit_transform(np.array(embedding_vectors))


fig, ax = plt.subplots(figsize=(8, 8))

cmap = matplotlib.colormaps["Spectral"]
color_stops = np.linspace(0, 1, len(display_labels))

LABEL_SHIFT = 1.5  # Adjust based on the data scale
label_positions = {}

for idx, label in enumerate(display_labels):
    cluster_points = tsne_results[cluster_labels == label]
    centroid = np.median(cluster_points, axis=0)
    color = cmap(color_stops[idx])
    ax.scatter(cluster_points[:, 0], cluster_points[:, 1], color=color)

    # Determine a non-overlapping label position
    y_position = centroid[1]
    while any(
        abs(y_position - other_y) <= LABEL_SHIFT for other_y in label_positions.values()
    ):
        y_position += LABEL_SHIFT  # Shift label up if there's an overlap
    label_positions[label] = y_position

    # Add the cluster label to the plot
    ax.text(
        centroid[0],
        y_position,
        f"{display_labels[label]}",
        horizontalalignment="center",
        verticalalignment="center",
        fontsize=8,
        weight="bold",
        color="black",
        bbox={
            "facecolor": color,
            "alpha": 0.5,
            "edgecolor": "none",
            "boxstyle": "round,pad=0.3",
        },
    )

ax.set_xlabel("t-SNE Dimension 1")
ax.set_ylabel("t-SNE Dimension 2")
ax.set_title(f"Cluster Visualization using t-SNE, clusters {args.clusters}")
ax.set_xticks([])
ax.set_yticks([])


fig.savefig(PLOT_FILENAME)
print(f"TSNE plot for saved as {PLOT_FILENAME}")
