"""
Wordclouds

This script is used to generate wordclouds for the dataset.

minh.ha-duong@cnrs.fr, 2024 CC-BY-SA
"""

import argparse
import re
import json
import pandas as pd
import matplotlib.pyplot as plt
from wordcloud import WordCloud, STOPWORDS


# Set up argument parser
parser = argparse.ArgumentParser(description="Generate t-SNE plot for clusters.")
parser.add_argument(
    "--clusters",
    type=str,
    choices=["auto", "manual"],
    required=True,
    help="Specify clustering mode: auto or manual",
)
args = parser.parse_args()

IN_FILENAME = "../dataset/CIRED_dataset.json"
CLUSTERS_FILENAME = f"../clusters_{args.clusters}/clusters.json"
FACETS_ANALYSIS_FILENAME = "../facets/facets_analysis.csv"
OUT_DIR = f"clusters_{args.clusters}/"

# Load facets data and merge with cluster IDs
df = pd.read_csv(FACETS_ANALYSIS_FILENAME)
df["Area"] = df["Area"].str.replace("-", "_", regex=False)
df["Time scope"] = df["Time scope"].str.replace("-", "_", regex=False)

# Load clusters from JSON file and add the column to the dataframe
with open(CLUSTERS_FILENAME, "r", encoding="utf-8") as file:
    clusters = json.load(file)
clusters = {int(k): v for k, v in clusters.items()}
clusters_df = pd.DataFrame(list(clusters.items()), columns=["docid", "cluster"])
df = df.merge(clusters_df, on="docid", how="left")

# Load the publication data from the JSON file
with open(IN_FILENAME, "r", encoding="utf-8") as file:
    publications_data = json.load(file)

# Add a column in the DataFrame with just the authors' last names
for pub in publications_data:
    if "authFullName_s" in pub:
        lastnames = " ".join(
            [name.split()[-1] for name in pub["authFullName_s"]]
        ).replace("-", "_")
        df.loc[df["docid"] == int(pub["docid"]), "Authors"] = lastnames

# Add a column in the DataFrame with the abstracts
publis = {
    int(pub["docid"]): pub["abstract_s"][0].lower()
    for pub in publications_data
    if "abstract_s" in pub
}
publis_df = pd.DataFrame(list(publis.items()), columns=["docid", "abstract"])
df = df.merge(publis_df, on="docid", how="left")
df = df.rename(columns={"abstract": "Abstract"})


# Add the All_discipline column
df["Disciplines"] = (
    df["Main discipline"].fillna("") + " " + df["Secondary discipline"].fillna("")
)

print(df)

# Add the All_SDG column
sdg_mapping = {
    1: "1_No_poverty",
    2: "2_Zero_hunger",
    3: "3_Good_health_and_well-being",
    4: "4_Quality_education",
    5: "5_Gender_equality",
    6: "6_Clean_water_and_sanitation",
    7: "7_Affordable_and_clean_energy",
    8: "8_Decent_work_and_economic_growth",
    9: "9_Industry_innovation_and_infrastructure",
    10: "10_Reduced_inequalities",
    11: "11_Sustainable_cities_and_communities",
    12: "12_Responsible_consumption_and_production",
    13: "13_Climate_action",
    14: "14_Life_below_water",
    15: "15_Life_on_land",
    16: "16_Peace_justice_and_strong_institutions",
    17: "17_Partnerships_for_the_goals",
}

df["Main SDG"] = df["Main SDG"].map(sdg_mapping).fillna("")
df["Other SDG"] = df["Other SDG"].map(sdg_mapping).fillna("")
df["Third SDG"] = df["Third SDG"].map(sdg_mapping).fillna("")
df["SDGs"] = df["Main SDG"] + " " + df["Other SDG"] + " " + df["Third SDG"]

# Define stopwords
excluded_words = set(STOPWORDS)
excluded_words.update(
    [
        "based",
        "new",
        "paper",
        "different",
        "use",
        "study",
        "show",
        "one",
        "two",
        "three",
        "system",
        "results",
        "used",
        "high",
        "low",
        "main",
        "using",
        "value",
        "issue",
        "large",
        "first",
        "chapter",
        "article",
        "method",
        "science",
        "studies",
        "model",
        "models",
        "e",
        "way",
        "researcher",
        "research",
    ]
)


def generate_word_clouds(df_subset, cluster_id, excluded_words):
    """Draw and save the figure, which has six wordclouds as subfigures.

    df_subset: A datafrane containing the words to be clouded.
    It must have columns: "Abstract", "Authors", "Area", "Time scope",
    "SDGs", "Disciplines".
    We do not draw the wordclouds of "Topic" and "Method", because
    the former is redundant with Abstract, the later not well defined.

    cluster_id: A string for display.
    The df_subset must be already properly filtered.
    """
    categories = ["Abstract", "Authors", "Area", "Time scope", "SDGs", "Disciplines"]
    fig, axs = plt.subplots(2, 3, figsize=(9, 4))
    axs = axs.flatten()
    assert len(categories) == len(
        axs
    ), "The number of categories must match the number of subplots"
    for idx, category in enumerate(categories):
        category_texts = df_subset[category].dropna().astype(str)
        text = " ".join(category_texts)
        wordcloud = WordCloud(
            background_color="white", stopwords=excluded_words
        ).generate(text)
        axs[idx].imshow(wordcloud, interpolation="bilinear")
        axs[idx].set_title(category)
        axs[idx].axis("off")

    plt.tight_layout()
    display_cluster_id = (
        (cluster_id[6:67] + "...") if len(str(cluster_id)) > 70 else cluster_id
    )
    num_abstracts = len(df_subset["Abstract"].dropna())
    plt.suptitle(f"{display_cluster_id} (n={num_abstracts})", size=16)
    fig.subplots_adjust(top=0.88)

    invalid_chars_pattern = r'[<>"/\\|?*]'  # We allow  :  in filename
    assert not re.search(
        invalid_chars_pattern, str(cluster_id)
    ), f"Error: The filename '{cluster_id}' contains invalid characters."
    plt.savefig(f"{OUT_DIR}{cluster_id}.png")
    plt.close(fig)


generate_word_clouds(df, "all", excluded_words)

# Create a word cloud for each cluster
cluster_ids = set(df["cluster"].dropna())
for cluster_id in cluster_ids:
    generate_word_clouds(df[df["cluster"] == cluster_id], cluster_id, excluded_words)
